/**
 * XmlSerializer Workshop Copyright (c) Simon Steele, 2003
 * see included license.txt for licensing.
 */
using System;
using System.Security.Policy;
using System.Runtime.InteropServices;
using System.IO;
using System.Reflection;
using System.Text.RegularExpressions;
using System.Xml.Serialization;

namespace XmlSerializerWorkshop
{
	public class TestInfo
	{
		public Type type = null;
		
		public bool Tested = false;
		public bool CanSerialize = false;
		public string message;
		public string fullText;
		public bool HasSourceAndData = false;
		public string outData;
		public string csSourcePath;
		public System.Reflection.Assembly assembly = null;
	}

	public class Abuser
	{
		Assembly theAssembly = null;
		Assembly theLoadedAssembly = null;

		/// <summary>Fired when an assembly is loaded due to XmlSerializer compilation,
		/// or when an attempt to create that assembly fails.</summary>
		public event XmlSerializerWorkshop.DomainGlue.AssemblyCreatedDelegate OnAssemblyCreated;

		/// <summary>Default Constructor.</summary>
		public Abuser()
		{
			AppDomain.CurrentDomain.AssemblyLoad += new AssemblyLoadEventHandler(OnAssemblyLoadEvent);
		}

		/// <summary>Array of types contained in the currently loaded assembly.</summary>
		public Type[] Types
		{
			get 
			{ 
				if(theAssembly == null) 
					throw new NullReferenceException("Must open an assembly first!"); 
				
				return theAssembly.GetTypes();
			}
		}

		/// <summary>Load an assembly to furkle with.</summary>
		/// <param name="assemblyFile">Assembly to load in the LoadFrom context.</param>
		/// <returns>True if successful.</returns>
		public bool LoadAssembly(string assemblyFile)
		{
			theAssembly = null;
			
			Assembly ass = System.Reflection.Assembly.LoadFrom( assemblyFile );
			if(ass == null)
				return false;

			theAssembly = ass;

			return true;
		}

		string buildExceptionText(InvalidOperationException theException)
		{
			string output = "";

			Exception ex = theException;
			while(ex != null)
			{
				if(output.Length > 0)
					output += "\n\n";
				output += ex.Message;
				
				ex = ex.InnerException;
			}

			return output;
		}

		string buildExceptionDetails(InvalidOperationException theException)
		{
			string output = "";

			Exception ex = theException;
			while(ex != null)
			{
				if(output.Length > 0)
					output += "\n\n";
				output += ex.ToString();
				
				ex = ex.InnerException;
			}

			return output;
		}

		/// <summary>See if a type can be serialized.</summary>
		/// <param name="info">TestInfo object for this type.</param>
		/// <returns>True if the type can be XmlSerialized.</returns>
		public bool AnalyseType(TestInfo info)
		{
			if( info.Tested )
				return info.CanSerialize;
			
			Type t = info.type;

			try
			{
				info.Tested = true;
				info.CanSerialize = true;

				theLoadedAssembly = null;

				XmlSerializer s = new XmlSerializer(t);

				if(theLoadedAssembly != null)
				{
					info.assembly = theLoadedAssembly;
					theLoadedAssembly = null;
				}
			}
			catch( NullReferenceException nre )
			{
				System.Windows.Forms.MessageBox.Show("NullReferenceException!!\n\n" + nre.ToString());
			}
			catch( InvalidOperationException iex )
			{
				info.CanSerialize = false;
				info.message = buildExceptionText(iex);
				info.fullText = buildExceptionDetails(iex);
			}
			catch( Exception ex )
			{
				info.CanSerialize = false;

				// From Chris Sells' RegEx PreCompiler ((c) Microsoft):

				Regex regex = new Regex(@"File or assembly name (?<baseFileName>.*).dll");
				Match match = regex.Match(ex.Message);
				string baseFileName = match.Groups["baseFileName"].Value;
				if( baseFileName.Length == 0 || baseFileName == string.Empty ) 
					return info.CanSerialize; // fail.

				OnAssemblyCreated(baseFileName, Path.GetTempPath());

				info.HasSourceAndData = true;

				string outputPath = Path.Combine(Path.GetTempPath(), baseFileName + ".out");
				info.outData = (new StreamReader(outputPath)).ReadToEnd();
					
				string csPath = Path.Combine(Path.GetTempPath(), baseFileName + ".0.cs");
				info.csSourcePath = csPath;
			}

			return info.CanSerialize;
		}

		/// <summary>
		/// Called when an assembly is loaded (i.e. an xml serializer compile product).
		/// We store the assembly to attach it to the test info for the current type.
		/// </summary>
		void OnAssemblyLoadEvent(object sender, AssemblyLoadEventArgs args) 
		{
			// Get the location of the assembly file.
			string loc = Path.GetDirectoryName(args.LoadedAssembly.Location);
			if(loc[loc.Length-1] != '\\')
				loc += '\\';
			loc = loc.ToLower();
		
			// Get the temp file path.
			string tempPath = Path.GetTempPath();
			tempPath = tempPath.ToLower();
			
			if(loc == tempPath)
			{
				System.Diagnostics.Debug.WriteLine("Loaded XmlSerializer Assembly: " + args.LoadedAssembly.FullName);
				theLoadedAssembly = args.LoadedAssembly;

				OnAssemblyCreated(args.LoadedAssembly.GetName().Name, loc);
			}
		}
	}

}